(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "./Circle", "./Rectangle", "../../Utils/NumberUtils"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.QuadTree = void 0;
    const Circle_1 = require("./Circle");
    const Rectangle_1 = require("./Rectangle");
    const NumberUtils_1 = require("../../Utils/NumberUtils");
    class QuadTree {
        constructor(rectangle, capacity) {
            this.rectangle = rectangle;
            this.capacity = capacity;
            this._points = [];
            this._divided = false;
        }
        insert(point) {
            var _a, _b, _c, _d, _e;
            if (!this.rectangle.contains(point.position)) {
                return false;
            }
            if (this._points.length < this.capacity) {
                this._points.push(point);
                return true;
            }
            if (!this._divided) {
                this.subdivide();
            }
            return ((_e = (((_a = this._NE) === null || _a === void 0 ? void 0 : _a.insert(point)) ||
                ((_b = this._NW) === null || _b === void 0 ? void 0 : _b.insert(point)) ||
                ((_c = this._SE) === null || _c === void 0 ? void 0 : _c.insert(point)) ||
                ((_d = this._SW) === null || _d === void 0 ? void 0 : _d.insert(point)))) !== null && _e !== void 0 ? _e : false);
        }
        query(range, check, found) {
            var _a, _b, _c, _d;
            const res = found !== null && found !== void 0 ? found : [];
            if (!range.intersects(this.rectangle)) {
                return [];
            }
            for (const p of this._points) {
                if (!range.contains(p.position) &&
                    (0, NumberUtils_1.getDistance)(range.position, p.position) > p.particle.getRadius() &&
                    (!check || check(p.particle))) {
                    continue;
                }
                res.push(p.particle);
            }
            if (this._divided) {
                (_a = this._NE) === null || _a === void 0 ? void 0 : _a.query(range, check, res);
                (_b = this._NW) === null || _b === void 0 ? void 0 : _b.query(range, check, res);
                (_c = this._SE) === null || _c === void 0 ? void 0 : _c.query(range, check, res);
                (_d = this._SW) === null || _d === void 0 ? void 0 : _d.query(range, check, res);
            }
            return res;
        }
        queryCircle(position, radius, check) {
            return this.query(new Circle_1.Circle(position.x, position.y, radius), check);
        }
        queryRectangle(position, size, check) {
            return this.query(new Rectangle_1.Rectangle(position.x, position.y, size.width, size.height), check);
        }
        subdivide() {
            const x = this.rectangle.position.x, y = this.rectangle.position.y, w = this.rectangle.size.width, h = this.rectangle.size.height, capacity = this.capacity;
            this._NE = new QuadTree(new Rectangle_1.Rectangle(x, y, w / 2, h / 2), capacity);
            this._NW = new QuadTree(new Rectangle_1.Rectangle(x + w / 2, y, w / 2, h / 2), capacity);
            this._SE = new QuadTree(new Rectangle_1.Rectangle(x, y + h / 2, w / 2, h / 2), capacity);
            this._SW = new QuadTree(new Rectangle_1.Rectangle(x + w / 2, y + h / 2, w / 2, h / 2), capacity);
            this._divided = true;
        }
    }
    exports.QuadTree = QuadTree;
});
